package com.tzld.piaoquan.api.controller;

import com.stuuudy.commons.external.filestorage.enums.EnumPublicBuckets;
import com.tzld.piaoquan.api.common.enums.ExceptionEnum;
import com.tzld.piaoquan.api.common.exception.CommonException;
import com.tzld.piaoquan.api.config.AliOssConfig;
import com.tzld.piaoquan.api.model.param.FileUploadParam;
import com.tzld.piaoquan.api.model.vo.FileInfo;
import com.tzld.piaoquan.api.util.AliOssFileTool;
import com.tzld.piaoquan.growth.common.common.base.CommonResponse;
import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;

@RestController
@RequestMapping("/file")
@CrossOrigin(origins = "*")
public class FileController {

    @PostMapping("/upload")
    public CommonResponse<FileInfo> fileUpload(@Validated FileUploadParam param, BindingResult bindingResult) throws IOException {
        if (bindingResult.hasErrors()) {
            return CommonResponse.create(ExceptionEnum.PARAM_ERROR.getCode(), bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        if (StringUtils.isBlank(param.getFileUri())) {
            String fileName = param.getFile().getOriginalFilename();
            if (StringUtils.isBlank(fileName)) {
                param.setFileUri("temp/" + System.currentTimeMillis());
            } else {
                param.setFileUri("temp/" + System.currentTimeMillis() + "_" + fileName.replace(" ", "_"));
            }
        }
        String fileUrl = AliOssFileTool.saveInPublicReturnHost(param.getFile().getInputStream(),
                EnumPublicBuckets.PUBBUCKET, param.getFileUri(), param.getFileType());
        FileInfo fileInfo = new FileInfo();
        fileInfo.setFileUrl(fileUrl);
        String bucketName = AliOssConfig.getBucket(EnumPublicBuckets.PUBBUCKET.getBucketName());
        boolean isExistFile = AliOssFileTool.getOssClient().doesObjectExist(bucketName, param.getFileUri());
        if (!isExistFile){
            throw new CommonException(ExceptionEnum.PARAM_ERROR.getCode(), "上传文件不存在！！");
        }
        return CommonResponse.create(fileInfo);
    }

}
